@@ -1,6 +1,6 @@
 import logging
 
-from sqeleton.databases import Connect
+from data_diff.sqeleton.databases import Connect
 
 from .postgresql import PostgreSQL
 from .mysql import MySQL
