@@ -1,4 +1,4 @@
-from typing import List, Dict
+from typing import List, Dict, Optional, Any, Tuple, Union
 
 from runtype import dataclass
 
@@ -9,14 +9,16 @@ from .table_segment import TableSegment
 class SegmentInfo:
     tables: List[TableSegment]
 
-    diff: list = None
+    diff: List[Union[Tuple[Any, ...], List[Any]]] = None
+    diff_schema: Tuple[Tuple[str, type], ...] = None
     is_diff: bool = None
     diff_count: int = None
 
     rowcounts: Dict[int, int] = {}
     max_rows: int = None
 
-    def set_diff(self, diff):
+    def set_diff(self, diff: List[Union[Tuple[Any, ...], List[Any]]], schema: Optional[Tuple[Tuple[str, type]]] = None):
+        self.diff_schema = schema
         self.diff = diff
         self.diff_count = len(diff)
         self.is_diff = self.diff_count > 0
@@ -28,6 +30,8 @@ class SegmentInfo:
         # self.diff = list(chain(*[c.diff for c in child_infos]))
         self.diff_count = sum(c.diff_count for c in child_infos if c.diff_count is not None)
         self.is_diff = any(c.is_diff for c in child_infos)
+        self.diff_schema = next((child.diff_schema for child in child_infos if child.diff_schema is not None), None)
+        self.diff = sum((c.diff for c in child_infos if c.diff is not None), [])
 
         self.rowcounts = {
             1: sum(c.rowcounts[1] for c in child_infos if c.rowcounts),
